@@ -1,17 +1,38 @@
 package aQute.libg.sed;
 
-import java.io.*;
-import java.lang.reflect.*;
-import java.net.*;
-import java.text.*;
-import java.util.*;
-import java.util.regex.*;
+import java.io.File;
+import java.io.IOException;
+import java.lang.reflect.Field;
+import java.lang.reflect.InvocationTargetException;
+import java.lang.reflect.Method;
+import java.net.MalformedURLException;
+import java.net.URL;
+import java.text.SimpleDateFormat;
+import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.Collection;
+import java.util.Collections;
+import java.util.Comparator;
+import java.util.Date;
+import java.util.Formatter;
+import java.util.HashMap;
+import java.util.Iterator;
+import java.util.LinkedHashSet;
+import java.util.List;
+import java.util.Map;
+import java.util.Properties;
+import java.util.Random;
+import java.util.Set;
+import java.util.TimeZone;
+import java.util.regex.Matcher;
+import java.util.regex.Pattern;
 
-import aQute.lib.collections.*;
-import aQute.lib.io.*;
-import aQute.libg.glob.*;
-import aQute.libg.reporter.*;
-import aQute.service.reporter.*;
+import aQute.lib.collections.ExtList;
+import aQute.lib.collections.SortedList;
+import aQute.lib.io.IO;
+import aQute.libg.glob.Glob;
+import aQute.libg.reporter.ReporterAdapter;
+import aQute.service.reporter.Reporter;
 
 /**
  * Provide a macro Domain. This Domain can replace variables in strings based on
@@ -118,10 +139,8 @@ public class ReplacerAdapter extends ReporterAdapter implements Replacer {
 	/**
 	 * Traverses a string to find a macro. It can handle nested brackets.
 	 * 
-	 * @param line
-	 *            The line with the macro
-	 * @param index
-	 *            Points to the character after the '$'
+	 * @param line The line with the macro
+	 * @param index Points to the character after the '$'
 	 * @return the end position
 	 */
 	public int findMacro(CharSequence line, int index) {
@@ -265,15 +284,14 @@ public class ReplacerAdapter extends ReporterAdapter implements Replacer {
 								value = process(template, new Link(domain, link, key));
 								if (value != null)
 									return value;
-							}
-							finally {
+							} finally {
 								domain = old;
 							}
 						}
 					}
 				}
 				if (!flattening && !key.equals("@"))
-					reporter.warning("No translation found for macro: " + key);
+					reporter.warning("No translation found for macro: %s", key);
 			} else {
 				reporter.warning("Found empty macro key");
 			}
@@ -298,11 +316,8 @@ public class ReplacerAdapter extends ReporterAdapter implements Replacer {
 	/**
 	 * Parse the key as a command. A command consist of parameters separated by
 	 * ':'.
-	 * 
-	 * @param key
-	 * @return
 	 */
-	static Pattern	commands	= Pattern.compile("(?<!\\\\);");
+	static Pattern commands = Pattern.compile("(?<!\\\\);");
 
 	private String doCommands(String key, Link source) {
 		String[] args = commands.split(key);
@@ -348,26 +363,23 @@ public class ReplacerAdapter extends ReporterAdapter implements Replacer {
 			String cname = "_" + method.replaceAll("-", "_");
 			try {
 				Method m = target.getClass().getMethod(cname, new Class[] {
-					String[].class
+						String[].class
 				});
 				return "" + m.invoke(target, new Object[] {
-					args
+						args
 				});
-			}
-			catch (NoSuchMethodException e) {
+			} catch (NoSuchMethodException e) {
 				// Ignore
-			}
-			catch (InvocationTargetException e) {
+			} catch (InvocationTargetException e) {
 				if (e.getCause() instanceof IllegalArgumentException) {
-					reporter.error("%s, for cmd: %s, arguments; %s", e.getCause().getMessage(), method,
+					reporter.error("%s, for cmd: %s, arguments; %s", e.getCause(), method,
 							Arrays.toString(args));
 				} else {
-					reporter.warning("Exception in replace: " + e.getCause());
+					reporter.warning("Exception in replace: %s", e.getCause());
 					e.getCause().printStackTrace();
 				}
-			}
-			catch (Exception e) {
-				reporter.warning("Exception in replace: " + e + " method=" + method);
+			} catch (Exception e) {
+				reporter.warning("Exception in replace: %s method=%s", e, method);
 				e.printStackTrace();
 			}
 		}
@@ -376,11 +388,8 @@ public class ReplacerAdapter extends ReporterAdapter implements Replacer {
 
 	/**
 	 * Return a unique list where the duplicates are removed.
-	 * 
-	 * @param args
-	 * @return
 	 */
-	static String	_uniqHelp	= "${uniq;<list> ...}";
+	static String _uniqHelp = "${uniq;<list> ...}";
 
 	public String _uniq(String args[]) {
 		verifyCommand(args, _uniqHelp, null, 1, Integer.MAX_VALUE);
@@ -410,7 +419,7 @@ public class ReplacerAdapter extends ReporterAdapter implements Replacer {
 
 	}
 
-	static String	_filterHelp	= "${%s;<list>;<regex>}";
+	static String _filterHelp = "${%s;<list>;<regex>}";
 
 	String filter(String[] args, boolean include) {
 		verifyCommand(args, String.format(_filterHelp, args[0]), null, 3, 3);
@@ -425,7 +434,7 @@ public class ReplacerAdapter extends ReporterAdapter implements Replacer {
 		return list.join();
 	}
 
-	static String	_sortHelp	= "${sort;<list>...}";
+	static String _sortHelp = "${sort;<list>...}";
 
 	public String _sort(String args[]) {
 		verifyCommand(args, _sortHelp, null, 2, Integer.MAX_VALUE);
@@ -438,7 +447,7 @@ public class ReplacerAdapter extends ReporterAdapter implements Replacer {
 		return result.join();
 	}
 
-	static String	_nsortHelp	= "${nsort;<list>...}";
+	static String _nsortHelp = "${nsort;<list>...}";
 
 	public String _nsort(String args[]) {
 		verifyCommand(args, _nsortHelp, null, 2, Integer.MAX_VALUE);
@@ -468,7 +477,7 @@ public class ReplacerAdapter extends ReporterAdapter implements Replacer {
 		return result.join();
 	}
 
-	static String	_joinHelp	= "${join;<list>...}";
+	static String _joinHelp = "${join;<list>...}";
 
 	public String _join(String args[]) {
 
@@ -481,7 +490,7 @@ public class ReplacerAdapter extends ReporterAdapter implements Replacer {
 		return result.join();
 	}
 
-	static String	_ifHelp	= "${if;<condition>;<iftrue> [;<iffalse>] }";
+	static String _ifHelp = "${if;<condition>;<iftrue> [;<iffalse>] }";
 
 	public String _if(String args[]) {
 		verifyCommand(args, _ifHelp, null, 3, 4);
@@ -499,7 +508,7 @@ public class ReplacerAdapter extends ReporterAdapter implements Replacer {
 		return new Date().toString();
 	}
 
-	public final static String	_fmodifiedHelp	= "${fmodified;<list of filenames>...}, return latest modification date";
+	public final static String _fmodifiedHelp = "${fmodified;<list of filenames>...}, return latest modification date";
 
 	public String _fmodified(String args[]) throws Exception {
 		verifyCommand(args, _fmodifiedHelp, null, 2, Integer.MAX_VALUE);
@@ -520,8 +529,7 @@ public class ReplacerAdapter extends ReporterAdapter implements Replacer {
 	public String _long2date(String args[]) {
 		try {
 			return new Date(Long.parseLong(args[1])).toString();
-		}
-		catch (Exception e) {
+		} catch (Exception e) {
 			e.printStackTrace();
 		}
 		return "not a valid long";
@@ -548,11 +556,11 @@ public class ReplacerAdapter extends ReporterAdapter implements Replacer {
 	 * replace ; <list> ; regex ; replace
 	 * 
 	 * @param args
-	 * @return
+	 * @return result
 	 */
 	public String _replace(String args[]) {
 		if (args.length != 4) {
-			reporter.warning("Invalid nr of arguments to replace " + Arrays.asList(args));
+			reporter.warning("Invalid nr of arguments to replace %s", Arrays.asList(args));
 			return null;
 		}
 
@@ -573,25 +581,22 @@ public class ReplacerAdapter extends ReporterAdapter implements Replacer {
 
 	public String _warning(String args[]) {
 		for (int i = 1; i < args.length; i++) {
-			reporter.warning(process(args[i]));
+			reporter.warning("%s", process(args[i]));
 		}
 		return "";
 	}
 
 	public String _error(String args[]) {
 		for (int i = 1; i < args.length; i++) {
-			reporter.error(process(args[i]));
+			reporter.error("%s", process(args[i]));
 		}
 		return "";
 	}
 
 	/**
 	 * toclassname ; <path>.class ( , <path>.class ) *
-	 * 
-	 * @param args
-	 * @return
 	 */
-	static String	_toclassnameHelp	= "${classname;<list of class names>}, convert class paths to FQN class names ";
+	static String _toclassnameHelp = "${classname;<list of class names>}, convert class paths to FQN class names ";
 
 	public String _toclassname(String args[]) {
 		verifyCommand(args, _toclassnameHelp, null, 2, 2);
@@ -614,12 +619,9 @@ public class ReplacerAdapter extends ReporterAdapter implements Replacer {
 
 	/**
 	 * toclassname ; <path>.class ( , <path>.class ) *
-	 * 
-	 * @param args
-	 * @return
 	 */
 
-	static String	_toclasspathHelp	= "${toclasspath;<list>[;boolean]}, convert a list of class names to paths";
+	static String _toclasspathHelp = "${toclasspath;<list>[;boolean]}, convert a list of class names to paths";
 
 	public String _toclasspath(String args[]) {
 		verifyCommand(args, _toclasspathHelp, null, 2, 3);
@@ -717,7 +719,7 @@ public class ReplacerAdapter extends ReporterAdapter implements Replacer {
 			now = Long.parseLong(args[3]);
 		}
 		if (args.length > 4) {
-			reporter.warning("Too many arguments for tstamp: " + Arrays.toString(args));
+			reporter.warning("Too many arguments for tstamp: %s", Arrays.toString(args));
 		}
 
 		SimpleDateFormat sdf = new SimpleDateFormat(format);
@@ -728,8 +730,8 @@ public class ReplacerAdapter extends ReporterAdapter implements Replacer {
 
 	/**
 	 * Wildcard a directory. The lists can contain Instruction that are matched
-	 * against the given directory ${lsr;<dir>;<list>(;<list>)*}
-	 * ${lsa;<dir>;<list>(;<list>)*}
+	 * against the given directory ${lsr;<dir>;<list>(;<list>)*} ${lsa;<dir>;
+	 * <list>(;<list>)*}
 	 * 
 	 * @author aqute
 	 */
@@ -766,7 +768,7 @@ public class ReplacerAdapter extends ReporterAdapter implements Replacer {
 
 		ExtList<String> result = new ExtList<String>();
 		for (File file : files)
-			result.add(relative ? file.getName() : file.getAbsolutePath());
+			result.add(relative ? file.getName() : file.getAbsolutePath().replace(File.separatorChar, '/'));
 
 		return result.join(",");
 	}
@@ -777,16 +779,11 @@ public class ReplacerAdapter extends ReporterAdapter implements Replacer {
 
 	/**
 	 * System command. Execute a command and insert the result.
-	 * 
-	 * @param args
-	 * @param help
-	 * @param patterns
-	 * @param low
-	 * @param high
 	 */
 	public String system_internal(boolean allowFail, String args[]) throws Exception {
-		verifyCommand(args, "${" + (allowFail ? "system-allow-fail" : "system")
-				+ ";<command>[;<in>]}, execute a system command", null, 2, 3);
+		verifyCommand(args,
+				"${" + (allowFail ? "system-allow-fail" : "system") + ";<command>[;<in>]}, execute a system command",
+				null, 2, 3);
 		String command = args[1];
 		String input = null;
 
@@ -808,9 +805,9 @@ public class ReplacerAdapter extends ReporterAdapter implements Replacer {
 
 		if (exitValue != 0) {
 			if (!allowFail) {
-				reporter.error("System command " + command + " failed with exit code " + exitValue);
+				reporter.error("System command %s failed with exit code %d", command, exitValue);
 			} else {
-				reporter.warning("System command " + command + " failed with exit code " + exitValue + " (allowed)");
+				reporter.warning("System command %s failed with exit code %d (allowed)", command, exitValue);
 
 			}
 		}
@@ -826,8 +823,7 @@ public class ReplacerAdapter extends ReporterAdapter implements Replacer {
 		String result = "";
 		try {
 			result = system_internal(true, args);
-		}
-		catch (Throwable t) {
+		} catch (Throwable t) {
 			/* ignore */
 		}
 		return result;
@@ -838,8 +834,7 @@ public class ReplacerAdapter extends ReporterAdapter implements Replacer {
 
 		try {
 			return System.getenv(args[1]);
-		}
-		catch (Throwable t) {
+		} catch (Throwable t) {
 			return null;
 		}
 	}
@@ -847,8 +842,7 @@ public class ReplacerAdapter extends ReporterAdapter implements Replacer {
 	/**
 	 * Get the contents of a file.
 	 * 
-	 * @param in
-	 * @return
+	 * @return contents of file
 	 * @throws IOException
 	 */
 
@@ -863,8 +857,7 @@ public class ReplacerAdapter extends ReporterAdapter implements Replacer {
 			try {
 				URL url = new URL(args[1]);
 				return IO.collect(url, "UTF-8");
-			}
-			catch (MalformedURLException mfue) {
+			} catch (MalformedURLException mfue) {
 				// Ignore here
 			}
 			return null;
@@ -964,13 +957,12 @@ public class ReplacerAdapter extends ReporterAdapter implements Replacer {
 						flattened.put(key, process(source.get(key)));
 			}
 			return flattened;
-		}
-		finally {
+		} finally {
 			flattening = false;
 		}
 	}
 
-	public final static String	_fileHelp	= "${file;<base>;<paths>...}, create correct OS dependent path";
+	public final static String _fileHelp = "${file;<base>;<paths>...}, create correct OS dependent path";
 
 	public String _osfile(String args[]) {
 		verifyCommand(args, _fileHelp, null, 3, 3);
@@ -992,8 +984,7 @@ public class ReplacerAdapter extends ReporterAdapter implements Replacer {
 			Field f = Properties.class.getDeclaredField("defaults");
 			f.setAccessible(true);
 			return (Properties) f.get(p);
-		}
-		catch (Exception e) {
+		} catch (Exception e) {
 			Field[] fields = Properties.class.getFields();
 			System.err.println(Arrays.toString(fields));
 			return null;
@@ -1015,8 +1006,7 @@ public class ReplacerAdapter extends ReporterAdapter implements Replacer {
 		if (args.length > 1) {
 			try {
 				numchars = Integer.parseInt(args[1]);
-			}
-			catch (NumberFormatException e) {
+			} catch (NumberFormatException e) {
 				throw new IllegalArgumentException("Invalid character count parameter in ${random} macro.");
 			}
 		}
@@ -1107,4 +1097,28 @@ public class ReplacerAdapter extends ReporterAdapter implements Replacer {
 		}
 		return sb.toString();
 	}
+
+	/**
+	 * Format bytes
+	 */
+
+	public String _bytes(String[] args) {
+		Formatter sb = new Formatter();
+		for (int i = 0; i < args.length; i++) {
+			long l = Long.parseLong(args[1]);
+			bytes(sb, l, 0, new String[] {
+					"b", "Kb", "Mb", "Gb", "Tb", "Pb", "Eb", "Zb", "Yb", "Bb", "Geopbyte"
+			});
+		}
+		return sb.toString();
+	}
+
+	private void bytes(Formatter sb, double l, int i, String[] strings) {
+		if (l > 1024 && i < strings.length - 1) {
+			bytes(sb, l / 1024, i + 1, strings);
+			return;
+		}
+		l = Math.round(l * 10) / 10;
+		sb.format("%s %s", l, strings[i]);
+	}
 }
